@@ -47,7 +47,6 @@ Provides:
 
 import itertools
 import src.lib.i18n as i18n
-import os
 import shutil
 
 try:
@@ -83,7 +82,7 @@ from src.actions._grid_cell_actions import CellActions
 
 from src.gui._events import post_command_event
 
-#use ugettext instead of getttext to avoid unicode errors
+# Use ugettext instead of getttext to avoid unicode errors
 _ = i18n.language.ugettext
 
 
@@ -179,6 +178,7 @@ class FileActions(Actions):
             return False
 
         # Check if the sig is valid for the sigfile
+        # TODO: Check for whitespace in filenpaths
         return verify(sigfilename, filename)
 
     def enter_safe_mode(self):
@@ -429,6 +429,147 @@ class FileActions(Actions):
             # The main window does not exist any more
             pass
 
+    def _set_save_states(self):
+        """Sets application save states"""
+
+        wx.BeginBusyCursor()
+        self.saving = True
+        self.grid.Disable()
+
+    def _release_save_states(self):
+        """Releases application save states"""
+
+        self.saving = False
+        self.grid.Enable()
+        wx.EndBusyCursor()
+
+        # Mark content as unchanged
+        try:
+            post_command_event(self.main_window, self.ContentChangedMsg,
+                               changed=False)
+        except TypeError:
+            # The main window does not exist any more
+            pass
+
+    def _move_tmp_file(self, tmpfilepath, filepath):
+        """Moves tmpfile over file after saving is finished
+
+        Parameters
+        ----------
+
+        filepath: String
+        \tTarget file path for xls file
+        tmpfilepath: String
+        \tTemporary file file path for xls file
+
+        """
+
+        try:
+            shutil.move(tmpfilepath, filepath)
+
+        except OSError, err:
+            # No tmp file present
+            post_command_event(self.main_window, self.StatusBarMsg, text=err)
+
+    def _save_xls(self, filepath):
+        """Saves file as xls workbook
+
+        Parameters
+        ----------
+
+        filepath: String
+        \tTarget file path for xls file
+
+        """
+
+        Interface = self.type2interface["xls"]
+
+        workbook = xlwt.Workbook()
+        interface = Interface(self.grid.code_array, workbook)
+        interface.from_code_array()
+
+        try:
+            workbook.save(filepath)
+
+        except IOError, err:
+            try:
+                post_command_event(self.main_window, self.StatusBarMsg,
+                                   text=err)
+            except TypeError:
+                # The main window does not exist any more
+                pass
+
+    def _save_pys(self, filepath):
+        """Saves file as pys file and returns True if save success
+
+        Parameters
+        ----------
+
+        filepath: String
+        \tTarget file path for xls file
+
+        """
+
+        try:
+            with Bz2AOpen(filepath, "wb",
+                          main_window=self.main_window) as outfile:
+                interface = Pys(self.grid.code_array, outfile)
+                interface.from_code_array()
+
+        except (IOError, ValueError), err:
+            try:
+                post_command_event(self.main_window, self.StatusBarMsg,
+                                   text=err)
+                return
+            except TypeError:
+                # The main window does not exist any more
+                pass
+
+        return not outfile.aborted
+
+    def _save_pysu(self, filepath):
+        """Saves file as pys file and returns True if save success
+
+        Parameters
+        ----------
+
+        filepath: String
+        \tTarget file path for xls file
+
+        """
+
+        try:
+            with AOpen(filepath, "wb",
+                       main_window=self.main_window) as outfile:
+                interface = Pys(self.grid.code_array, outfile)
+                interface.from_code_array()
+
+        except (IOError, ValueError), err:
+            try:
+                post_command_event(self.main_window, self.StatusBarMsg,
+                                   text=err)
+                return
+            except TypeError:
+                # The main window does not exist any more
+                pass
+
+        return not outfile.aborted
+
+    def _save_sign(self, filepath):
+        """Sign so that the new file may be retrieved without safe mode"""
+
+        if self.code_array.safe_mode:
+            msg = _("File saved but not signed because it is unapproved.")
+            try:
+                post_command_event(self.main_window, self.StatusBarMsg,
+                                   text=msg)
+            except TypeError:
+                # The main window does not exist any more
+                pass
+
+        else:
+            self.sign_file(filepath)
+
     def save(self, event):
         """Saves a file that is specified in event.attr
 
@@ -443,94 +584,38 @@ class FileActions(Actions):
 
         try:
             filetype = event.attr["filetype"]
-
         except KeyError:
             filetype = "pys"
 
-        Interface = self.type2interface[filetype]
+        # Use ntmpfile to make sure that old save file does not get lost
+        # on abort save
+        tmpfilepath = filepath + "~"
 
-        io_error_text = _("Error writing to file {filepath}.")
-        io_error_text = io_error_text.format(filepath=filepath)
+        if filetype == "xls":
+            self._set_save_states()
+            self._save_xls(tmpfilepath)
+            self._move_tmp_file(tmpfilepath, filepath)
+            self._release_save_states()
 
-        # Set state to file saving
-        self.saving = True
+        elif filetype == "pys":
+            self._set_save_states()
+            if self._save_pys(tmpfilepath):
+                # Writing was successful
+                self._move_tmp_file(tmpfilepath, filepath)
+                self._save_sign(filepath)
+            self._release_save_states()
 
-        # Make sure that old save file does not get lost on abort save
-        tmpfile = filepath + "~"
+        elif filetype == "pysu":
+            self._set_save_states()
+            if self._save_pysu(tmpfilepath):
+                # Writing was successful
+                self._move_tmp_file(tmpfilepath, filepath)
+                self._save_sign(filepath)
+            self._release_save_states()
 
-        type2opener = {
-            "pys":
-            (Bz2AOpen, [tmpfile, "wb"], {"main_window": self.main_window}),
-            "pysu":
-            (AOpen, [tmpfile, "wb"], {"main_window": self.main_window}),
-        }
-
-        try:
-            wx.BeginBusyCursor()
-            self.grid.Disable()
-
-            if filetype == "xls":
-                workbook = xlwt.Workbook()
-                interface = Interface(self.grid.code_array, workbook)
-                interface.from_code_array()
-                workbook.save(tmpfile)
-
-            else:
-                # Specify the interface that shall be used
-                opener, op_args, op_kwargs = type2opener[filetype]
-
-                with opener(*op_args, **op_kwargs) as outfile:
-                    try:
-                        interface = Interface(self.grid.code_array, outfile)
-                        interface.from_code_array()
-
-                    except ValueError, err:
-                        post_command_event(self.main_window, self.StatusBarMsg,
-                                           text=err)
-
-            # Move save file from temp file to filepath
-            try:
-                shutil.move(tmpfile, filepath)
-
-            except OSError:
-                # No tmp file present
-                pass
-
-        except IOError:
-            try:
-                post_command_event(self.main_window, self.StatusBarMsg,
-                                   text=io_error_text)
-            except TypeError:
-                # The main window does not exist any more
-                pass
-
-            return False
-
-        finally:
-            self.saving = False
-            self.grid.Enable()
-            wx.EndBusyCursor()
-
-        # Mark content as unchanged
-        try:
-            post_command_event(self.main_window, self.ContentChangedMsg,
-                               changed=False)
-        except TypeError:
-            # The main window does not exist any more
-            pass
-
-        # Sign so that the new file may be retrieved without safe mode
-        if self.code_array.safe_mode:
-            msg = _("File saved but not signed because it is unapproved.")
-            try:
-                post_command_event(self.main_window, self.StatusBarMsg,
-                                   text=msg)
-            except TypeError:
-                # The main window does not exist any more
-                pass
-
-        elif filetype in ["pys", "pysu"] and not outfile.aborted:
-            self.sign_file(filepath)
+        else:
+            msg = "Filetype {filetype} unknown.".format(filetype=filetype)
+            raise ValueError(msg)
 
 
 class TableRowActionsMixin(Actions):
@@ -1031,7 +1116,8 @@ class GridActions(Actions):
         self.grid.SetRowLabelSize(self.grid.row_label_size * zoom)
 
         for row, tab in self.code_array.row_heights:
-            if tab == self.grid.current_table:
+            if tab == self.grid.current_table and \
+               row < self.grid.code_array.shape[0]:
                 zoomed_row_size = \
                     self.code_array.row_heights[(row, tab)] * zoom
                 self.grid.SetRowSize(row, zoomed_row_size)
@@ -1044,7 +1130,8 @@ class GridActions(Actions):
         self.grid.SetColLabelSize(self.grid.col_label_size * zoom)
 
         for col, tab in self.code_array.col_widths:
-            if tab == self.grid.current_table:
+            if tab == self.grid.current_table and \
+               col < self.grid.code_array.shape[1]:
                 zoomed_col_size = self.code_array.col_widths[(col, tab)] * zoom
                 self.grid.SetColSize(col, zoomed_col_size)
 
