@@ -48,7 +48,10 @@ import os
 import wx
 import wx.html
 
-from matplotlib.figure import Figure
+try:
+    from matplotlib.figure import Figure
+except ImportError:
+    Figure = None
 
 import src.lib.i18n as i18n
 from src.sysvars import get_help_path
@@ -56,10 +59,18 @@ from src.sysvars import get_help_path
 from src.config import config
 from src.lib.__csv import CsvInterface, TxtGenerator
 from src.lib.charts import fig2bmp, fig2x
-from src.gui._printout import PrintCanvas, Printout
+from src.gui._printout import Printout
 from src.gui._events import post_command_event, EventMixin
+from src.lib._grid_cairo_renderer import GridCairoRenderer
 
-#use ugettext instead of getttext to avoid unicode errors
+try:
+    import cairo
+    HAS_CAIRO = True
+
+except ImportError:
+    HAS_CAIRO = False
+
+# use ugettext instead of getttext to avoid unicode errors
 _ = i18n.language.ugettext
 
 
@@ -207,28 +218,106 @@ class ExchangeActions(Actions):
         finally:
             outfile.close()
 
-    def export_file(self, filepath, filterindex, data, preview_data=None):
+    def export_file(self, filepath, __filter, data, preview_data=None):
         """Export data for other applications
 
         Parameters
         ----------
         filepath: String
         \tPath of export file
-        filterindex: Integer
-        \tIndex of the import filter
+        __filter: String
+        \tImport filter
         data: Object
         \tCode array result object slice, i. e. one object or iterable of
         \tsuch objects
 
         """
 
-        formats = ["csv", "svg", "eps", "ps", "pdf", "png"]
+        if __filter.startswith("cell_"):
+            self._export_figure(filepath, data, __filter[5:])
 
-        if filterindex == 0:
+        elif __filter == "csv":
             self._export_csv(filepath, data, preview_data=preview_data)
 
-        elif filterindex >= 1:
-            self._export_figure(filepath, data, formats[filterindex])
+        elif __filter in ["pdf", "svg"]:
+            self.export_cairo(filepath, __filter)
+
+    def get_print_rect(self, grid_rect):
+        """Returns wx.Rect that is correctly positioned on the print canvas"""
+
+        grid = self.grid
+
+        rect_x = grid_rect.x - \
+            grid.GetScrollPos(wx.HORIZONTAL) * grid.GetScrollLineX()
+        rect_y = grid_rect.y - \
+            grid.GetScrollPos(wx.VERTICAL) * grid.GetScrollLineY()
+
+        return wx.Rect(rect_x, rect_y, grid_rect.width, grid_rect.height)
+
+    def export_cairo(self, filepath, filetype):
+        """Exports grid to the PDF file filepath
+
+        Parameters
+        ----------
+        filepath: String
+        \tPath of file to export
+        filetype in ["pdf", "svg"]
+        \tType of file to export
+
+        """
+
+        if not HAS_CAIRO:
+            return
+
+        export_info = \
+            self.main_window.interfaces.get_cairo_export_info(filetype)
+
+        if export_info is None:
+            # Dialog has been canceled
+            return
+
+        top_row = max(0, export_info["top_row"])
+        bottom_row = min(self.grid.code_array.shape[0] - 1,
+                         export_info["bottom_row"])
+        left_col = max(0, export_info["left_col"])
+        right_col = min(self.grid.code_array.shape[1] - 1,
+                        export_info["right_col"])
+        first_tab = max(0, export_info["first_tab"])
+        last_tab = min(self.grid.code_array.shape[2] - 1,
+                       export_info["last_tab"])
+        width = export_info["paper_width"]
+        height = export_info["paper_height"]
+        orientation = export_info["orientation"]
+
+        if orientation == "landscape":
+            width, height = height, width
+
+        if filetype == "pdf":
+            surface = cairo.PDFSurface(filepath, width, height)
+        elif filetype == "svg":
+            surface = cairo.SVGSurface(filepath, width, height)
+        else:
+            msg = "Export filetype {filtype} not supported.".format(
+                filetype=filetype)
+            raise ValueError(msg)
+
+        context = cairo.Context(surface)
+
+        grid_cairo_renderer = GridCairoRenderer(
+            context,
+            self.code_array,
+            (top_row, bottom_row + 1),
+            (left_col, right_col + 1),
+            (first_tab, last_tab + 1),
+            width,
+            height,
+            orientation,
+        )
+
+        grid_cairo_renderer.draw()
+
+        # Finish is required for matplotlib figures
+        surface.finish()
 
 
 class PrintActions(Actions):
@@ -237,16 +326,24 @@ class PrintActions(Actions):
     def print_preview(self, print_area, print_data):
         """Launch print preview"""
 
-        # Create the print canvas
-        canvas = PrintCanvas(self.main_window, self.grid, print_area)
+        if not HAS_CAIRO:
+            return
 
-        printout = Printout(canvas)
-        printout2 = Printout(canvas)
+        print_info = \
+            self.main_window.interfaces.get_cairo_export_info("Print")
 
-        preview = wx.PrintPreview(printout, printout2, print_data)
+        if print_info is None:
+            # Dialog has been canceled
+            return
+
+        printout_preview = Printout(self.grid, print_data, print_info)
+        printout_printing = Printout(self.grid, print_data, print_info)
+
+        preview = wx.PrintPreview(printout_preview, printout_printing,
+                                  print_data)
 
         if not preview.Ok():
-            print "Printout preview failed.\n"
+            # Printout preview failed
             return
 
         pfrm = wx.PreviewFrame(preview, self.main_window, _("Print preview"))
@@ -264,20 +361,23 @@ class PrintActions(Actions):
 
         """
 
+        print_info = \
+            self.main_window.interfaces.get_cairo_export_info("Print")
+
+        if print_info is None:
+            # Dialog has been canceled
+            return
+
         pdd = wx.PrintDialogData(print_data)
         printer = wx.Printer(pdd)
 
-        # Create the print canvas
-        canvas = PrintCanvas(self.main_window, self.grid, print_area)
-
-        printout = Printout(canvas)
+        printout = Printout(self.grid, print_data, print_info)
 
         if printer.Print(self.main_window, printout, True):
             self.print_data = \
                 wx.PrintData(printer.GetPrintDialogData().GetPrintData())
 
         printout.Destroy()
-        canvas.Destroy()
 
 
 class ClipboardActions(Actions):
@@ -435,7 +535,7 @@ class ClipboardActions(Actions):
                 # The result is a wx.Bitmap. Return it.
                 return result
 
-            elif isinstance(result, Figure):
+            elif Figure is not None and isinstance(result, Figure):
                 # The result is a matplotlib figure
                 # Therefore, a wx.Bitmap is returned
                 key = bb_top, bb_left, tab
@@ -724,6 +824,10 @@ class HelpActions(Actions):
         self.help_window.Bind(wx.EVT_MOVE, self.OnHelpMove)
         self.help_window.Bind(wx.EVT_SIZE, self.OnHelpSize)
         self.help_htmlwindow.Bind(wx.EVT_RIGHT_DOWN, self.OnHelpBack)
+        self.help_htmlwindow.Bind(wx.html.EVT_HTML_LINK_CLICKED,
+                                  lambda e: self.open_external_links(e))
+        self.help_htmlwindow.Bind(wx.EVT_MOUSEWHEEL,
+                                  lambda e: self.zoom_html(e))
 
         # Get help data
         current_path = os.getcwd()
@@ -767,6 +871,57 @@ class HelpActions(Actions):
 
         event.Skip()
 
+    def open_external_links(self, event):
+        link = event.GetLinkInfo().GetHref()
+        if ':' in link:
+            wx.LaunchDefaultBrowser(link)
+        else:
+            self.help_htmlwindow.LoadPage(link)
+
+    def zoom_html(self, event):
+        if event.ControlDown():
+
+            if not hasattr(self, 'html_font_size'):
+                self.html_font_size = \
+                    self.help_htmlwindow.GetFont().GetPointSize()
+
+            # scroll down, zoom out
+            if event.GetWheelRotation() < 0:
+
+                # no difference between i and -i, so 1 is the smallest value
+                if self.html_font_size == 1:
+                    return
+
+                if 0 < self.html_font_size <= 12:
+                    step = 1
+                elif 12 < self.html_font_size <= 24:
+                    step = 2
+                elif 24 < self.html_font_size <= 36:
+                    step = 4
+                elif 36 < self.html_font_size:
+                    step = 6
+
+                self.html_font_size -= step
+
+            # scroll up, zoom in
+            if event.GetWheelRotation() > 0:
+
+                if 1 <= self.html_font_size < 12:
+                    step = 1
+                elif 12 <= self.html_font_size < 24:
+                    step = 2
+                elif 24 <= self.html_font_size < 36:
+                    step = 4
+                elif 36 <= self.html_font_size:
+                    step = 6
+
+                self.html_font_size += step
+
+            self.help_htmlwindow.SetStandardFonts(size=self.html_font_size)
+
+        else:
+            event.Skip()
+
 
 class AllMainWindowActions(ExchangeActions, PrintActions,
                            ClipboardActions, MacroActions, HelpActions):
