@@ -29,7 +29,7 @@ from ast import literal_eval
 
 import wx
 
-VERSION = "0.3.3"
+VERSION = "0.4"
 
 
 class DefaultConfig(object):
@@ -75,6 +75,10 @@ class DefaultConfig(object):
 
         self.timer_interval = "1000"
 
+        # Default row height and col width e.g. for Cairo rendering
+        self.default_row_height = "23"
+        self.default_col_width = "80"
+
         # Maximum result length in a cell in characters
         self.max_result_length = "100000"
 
@@ -158,6 +162,11 @@ class Config(object):
 
             return literal_eval(getattr(self.data, key))
 
+        except SyntaxError:
+            # May happen if a file is not present any more
+
+            return None
+
     def __setitem__(self, key, value):
         """Main config element write access"""
 
