@@ -82,7 +82,7 @@ from src.lib.parsers import unquote_string
 from icons import icons
 from sysvars import get_default_font, get_color
 
-#use ugettext instead of getttext to avoid unicode errors
+# Use ugettext instead of getttext to avoid unicode errors
 _ = i18n.language.ugettext
 
 
@@ -348,7 +348,6 @@ class TextEditor(wx.Panel, ChartDialogEventMixin):
         for key in kwargs:
             if key == "color":
                 color = code2color(kwargs[key])
-                self.colorselect.SetValue(color)
                 self.colorselect.SetOwnForegroundColour(color)
 
             elif key == "fontname":
@@ -1541,6 +1540,8 @@ class ChartDialog(wx.Dialog, ChartDialogEventMixin):
         style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER | wx.THICK_FRAME
         wx.Dialog.__init__(self, main_window, -1, style=style)
 
+        self.updating = False
+
         self.grid = main_window.grid
         self.key = key
 
@@ -1623,7 +1624,6 @@ class ChartDialog(wx.Dialog, ChartDialogEventMixin):
         self.Layout()
         self.SetSizerAndFit(main_sizer)
 
-
     def get_figure(self, code):
         """Returns figure from executing code in grid
 
@@ -1813,4 +1813,9 @@ class ChartDialog(wx.Dialog, ChartDialogEventMixin):
     def OnUpdateFigurePanel(self, event):
         """Redraw event handler for the figure panel"""
 
+        if self.updating:
+            return
+
+        self.updating = True
         self.figure_panel.update(self.get_figure(self.code))
+        self.updating = False
