@@ -67,6 +67,7 @@ class MainWindowEventMixin(object):
     SafeModeExitMsg, EVT_CMD_SAFE_MODE_EXIT = new_command_event()
 
     PreferencesMsg, EVT_CMD_PREFERENCES = new_command_event()
+    NewGpgKeyMsg, EVT_CMD_NEW_GPG_KEY = new_command_event()
 
     CloseMsg, EVT_CMD_CLOSE = new_command_event()
 
@@ -89,8 +90,9 @@ class MainWindowEventMixin(object):
 
     MainToolbarToggleMsg, EVT_CMD_MAINTOOLBAR_TOGGLE = new_command_event()
     MacroToolbarToggleMsg, EVT_CMD_MACROTOOLBAR_TOGGLE = new_command_event()
+    WidgetToolbarToggleMsg, EVT_CMD_WIDGETTOOLBAR_TOGGLE = new_command_event()
     AttributesToolbarToggleMsg, EVT_CMD_ATTRIBUTESTOOLBAR_TOGGLE = \
-                                            new_command_event()
+        new_command_event()
     FindToolbarToggleMsg, EVT_CMD_FIND_TOOLBAR_TOGGLE = new_command_event()
     EntryLineToggleMsg, EVT_CMD_ENTRYLINE_TOGGLE = new_command_event()
     TableChoiceToggleMsg, EVT_CMD_TABLECHOICE_TOGGLE = new_command_event()
@@ -117,6 +119,7 @@ class GridCellEventMixin(object):
     FontStrikethroughMsg, EVT_CMD_FONTSTRIKETHROUGH = new_command_event()
     FrozenMsg, EVT_CMD_FROZEN = new_command_event()
     LockMsg, EVT_CMD_LOCK = new_command_event()
+    MarkupMsg, EVT_CMD_MARKUP = new_command_event()
     MergeMsg, EVT_CMD_MERGE = new_command_event()
     JustificationMsg, EVT_CMD_JUSTIFICATION = new_command_event()
     AlignmentMsg, EVT_CMD_ALIGNMENT = new_command_event()
@@ -127,6 +130,7 @@ class GridCellEventMixin(object):
     TextColorMsg, EVT_CMD_TEXTCOLOR = new_command_event()
     RotationDialogMsg,  EVT_CMD_ROTATIONDIALOG = new_command_event()
     TextRotationMsg, EVT_CMD_TEXTROTATATION = new_command_event()
+    ButtonCellMsg, EVT_CMD_BUTTON_CELL = new_command_event()
 
     # Cell edit events
 
@@ -147,6 +151,7 @@ class GridEventMixin(object):
     OpenMsg, EVT_CMD_OPEN = new_command_event()
     SaveMsg, EVT_CMD_SAVE = new_command_event()
     SaveAsMsg, EVT_CMD_SAVEAS = new_command_event()
+    ExportPDFMsg, EVT_CMD_EXPORT_PDF = new_command_event()
     ImportMsg, EVT_CMD_IMPORT = new_command_event()
     ExportMsg, EVT_CMD_EXPORT = new_command_event()
     ApproveMsg, EVT_CMD_APPROVE = new_command_event()
