@@ -32,6 +32,12 @@ Provides
 
 import wx.grid
 
+try:
+    import rsvg
+    from src.lib.parsers import is_svg
+except ImportError:
+    rsvg = None
+
 from _events import post_command_event, EventMixin
 
 from _grid_table import GridTable
@@ -50,7 +56,7 @@ from src.model.model import CodeArray
 from src.actions._grid_actions import AllGridActions
 from src.gui._grid_cell_editor import GridCellEditor
 
-#use ugettext instead of getttext to avoid unicode errors
+# Use ugettext instead of getttext to avoid unicode errors
 _ = i18n.language.ugettext
 
 
@@ -195,6 +201,8 @@ class Grid(wx.grid.Grid, EventMixin):
                          c_handlers.OnCellFontStrikethrough)
         main_window.Bind(self.EVT_CMD_FROZEN, c_handlers.OnCellFrozen)
         main_window.Bind(self.EVT_CMD_LOCK, c_handlers.OnCellLocked)
+        main_window.Bind(self.EVT_CMD_BUTTON_CELL, c_handlers.OnButtonCell)
+        main_window.Bind(self.EVT_CMD_MARKUP, c_handlers.OnCellMarkup)
         main_window.Bind(self.EVT_CMD_MERGE, c_handlers.OnMerge)
         main_window.Bind(self.EVT_CMD_JUSTIFICATION,
                          c_handlers.OnCellJustification)
@@ -393,28 +401,50 @@ class GridCellEventHandlers(object):
     def OnInsertBitmap(self, event):
         """Insert bitmap event handler"""
 
+        key = self.grid.actions.cursor
         # Get file name
-        wildcard = "*"
-        message = _("Select bitmap for current cell")
+
+        wildcard = _("Bitmap file") + " (*)|*"
+        if rsvg is not None:
+            wildcard += "|"+ _("SVG file") + " (*.svg)|*.svg"
+
+        message = _("Select image for current cell")
         style = wx.OPEN | wx.CHANGE_DIR
-        filepath, __ = \
+        filepath, index = \
             self.grid.interfaces.get_filepath_findex_from_user(wildcard,
                                                                message, style)
+        if index == 0:
+            # Bitmap loaded
+            try:
+                img = wx.EmptyImage(1, 1)
+                img.LoadFile(filepath)
+            except TypeError:
+                return
 
-        try:
-            img = wx.EmptyImage(1, 1)
-            img.LoadFile(filepath)
-        except TypeError:
-            return
+            if img.GetSize() == (-1, -1):
+                # Bitmap could not be read
+                return
 
-        if img.GetSize() == (-1, -1):
-            # Bitmap could not be read
-            return
+            code = self.grid.main_window.actions.img2code(key, img)
 
-        key = self.grid.actions.cursor
-        code = self.grid.main_window.actions.img2code(key, img)
+        elif index == 1 and rsvg is not None:
+            # SVG loaded
+            with open(filepath) as infile:
+                try:
+                    code = infile.read()
+                except IOError:
+                    return
+                if is_svg(code):
+                    code = 'u"""' + code + '"""'
+                else:
+                    # Does not seem to be an svg file
+                    return
 
-        self.grid.actions.set_code(key, code)
+        else:
+            code = None
+
+        if code:
+            self.grid.actions.set_code(key, code)
 
     def OnLinkBitmap(self, event):
         """Link bitmap event handler"""
@@ -569,6 +599,31 @@ class GridCellEventHandlers(object):
 
         event.Skip()
 
+    def OnButtonCell(self, event):
+        """Button cell event handler"""
+
+        # The button text
+        text = event.text
+
+        self.grid.actions.set_attr("button_cell", text, mark_unredo=False)
+
+        self.grid.ForceRefresh()
+
+        self.grid.update_attribute_toolbar()
+
+        event.Skip()
+
+    def OnCellMarkup(self, event):
+        """Cell markup event handler"""
+
+        self.grid.actions.toggle_attr("markup")
+
+        self.grid.ForceRefresh()
+
+        self.grid.update_attribute_toolbar()
+
+        event.Skip()
+
     def OnMerge(self, event):
         """Merge cells event handler"""
 
@@ -664,7 +719,7 @@ class GridCellEventHandlers(object):
     def OnCellTextRotation(self, event):
         """Cell text rotation event handler"""
 
-        self.grid.actions.set_attr("angle", event.angle, mark_unredo=True)
+        self.grid.actions.toggle_attr("angle")
 
         self.grid.ForceRefresh()
 
@@ -695,10 +750,16 @@ class GridCellEventHandlers(object):
             top, left, bottom, right = merge_area
             if self.grid._last_selected_cell == (top, left, tab):
                 if row == top + 1:
-                    self.grid.actions.set_cursor((bottom + 1, left, tab))
+                    if bottom + 1 < self.grid.code_array.shape[0]:
+                        self.grid.actions.set_cursor((bottom + 1, left, tab))
+                    else:
+                        self.grid.actions.set_cursor((top, left, tab))
                     return
                 elif col == left + 1:
-                    self.grid.actions.set_cursor((top, right + 1, tab))
+                    if right + 1 < self.grid.code_array.shape[1]:
+                        self.grid.actions.set_cursor((top, right + 1, tab))
+                    else:
+                        self.grid.actions.set_cursor((top, left, tab))
                     return
             elif (row, col) != (top, left):
                 self.grid.actions.set_cursor((top, left, tab))
@@ -785,7 +846,6 @@ class GridEventHandlers(object):
         }
 
         keycode = event.GetKeyCode()
-        #print keycode
 
         modifier = shift * event.ShiftDown() | \
             alt * event.AltDown() | ctrl * event.ControlDown()
@@ -1256,11 +1316,11 @@ class GridEventHandlers(object):
         for row in rows:
             self.grid.code_array.set_row_height(row, tab, rowsize,
                                                 mark_unredo=False)
-            self.grid.SetRowSize(row, rowsize)
+            self.grid.SetRowSize(row, rowsize * self.grid.grid_renderer.zoom)
         self.grid.code_array.unredo.mark()
 
         event.Skip()
-        self.grid.Refresh()
+        self.grid.ForceRefresh()
 
     def OnColSize(self, event):
         """Column size event handler"""
@@ -1286,11 +1346,11 @@ class GridEventHandlers(object):
         for col in cols:
             self.grid.code_array.set_col_width(col, tab, colsize,
                                                mark_unredo=False)
-            self.grid.SetColSize(col, colsize)
+            self.grid.SetColSize(col, colsize * self.grid.grid_renderer.zoom)
         self.grid.code_array.unredo.mark()
 
         event.Skip()
-        self.grid.Refresh()
+        self.grid.ForceRefresh()
 
     def OnSortAscending(self, event):
         """Sort ascending event handler"""
