@@ -35,7 +35,25 @@ import os
 import wx
 import wx.lib.agw.aui as aui
 
-from matplotlib.figure import Figure
+try:
+    from matplotlib.figure import Figure
+except ImportError:
+    Figure = None
+
+try:
+    import cairo
+except ImportError:
+    cairo = None
+
+try:
+    import xlrd
+except ImportError:
+    xlrd = None
+
+try:
+    from src.lib.gpg import genkey
+except ImportError:
+    genkey = None
 
 import src.lib.i18n as i18n
 from src.config import config
@@ -43,6 +61,7 @@ from src.sysvars import get_python_tutorial_path, is_gtk
 
 from _menubars import MainMenu
 from _toolbars import MainToolbar, MacroToolbar, FindToolbar, AttributesToolbar
+from _toolbars import WidgetToolbar
 from _widgets import EntryLineToolbarPanel, StatusBar
 
 from src.lib.clipboard import Clipboard
@@ -56,7 +75,7 @@ from _events import post_command_event, EventMixin
 
 from src.actions._main_window_actions import AllMainWindowActions
 
-#use ugettext instead of getttext to avoid unicode errors
+# Use ugettext instead of getttext to avoid unicode errors
 _ = i18n.language.ugettext
 
 
@@ -70,6 +89,15 @@ class MainWindow(wx.Frame, EventMixin):
         except KeyError:
             S = None
 
+        try:
+            dimensions = kwargs.pop("dimensions")
+
+        except KeyError:
+            dimensions = (
+                config["grid_rows"],
+                config["grid_columns"],
+                config["grid_tables"])
+
         wx.Frame.__init__(self, parent, *args, **kwargs)
 
         self.interfaces = GuiInterfaces(self)
@@ -78,9 +106,9 @@ class MainWindow(wx.Frame, EventMixin):
             self._mgr = aui.AuiManager(self)
 
         except Exception:
-            # This may fail if py.testv runs under Windows
+            # This may fail if py.test runs under Windows
             # Therefore, we set up a basic framework for the unit tests
-            self.grid = Grid(self, -1, S=S, dimensions=(1000, 100, 3))
+            self.grid = Grid(self, -1, S=S, dimensions=dimensions)
             self.clipboard = Clipboard()
             self.actions = AllMainWindowActions(self.grid)
 
@@ -116,18 +144,12 @@ class MainWindow(wx.Frame, EventMixin):
         self.macro_toolbar = MacroToolbar(self, -1)
         self.find_toolbar = FindToolbar(self, -1)
         self.attributes_toolbar = AttributesToolbar(self, -1)
+        self.widget_toolbar = WidgetToolbar(self, -1)
 
         # Entry line
         self.entry_line_panel = EntryLineToolbarPanel(self, -1)
 
         # Main grid
-
-        dimensions = (
-            config["grid_rows"],
-            config["grid_columns"],
-            config["grid_tables"],
-        )
-
         self.grid = Grid(self, -1, S=S, dimensions=dimensions)
 
         # Clipboard
@@ -161,12 +183,8 @@ class MainWindow(wx.Frame, EventMixin):
 
         self.set_icon(icons["PyspreadLogo"])
 
-        # Set initial size to 90% of screen
-        self.SetInitialSize(config["window_size"])
-        self.SetPosition(config["window_position"])
-
         # Without minimum size, initial size is minimum size in wxGTK
-        self.SetMinSize((2, 2))
+        self.minSizeSet = False
 
         # Leave save mode
         post_command_event(self, self.SafeModeExitMsg)
@@ -180,6 +198,7 @@ class MainWindow(wx.Frame, EventMixin):
             (self.attributes_toolbar, "attributes_toolbar",
              _("Format toolbar")),
             (self.find_toolbar, "find_toolbar", _("Find toolbar")),
+            (self.widget_toolbar, "widget_toolbar", _("Widget toolbar")),
             (self.entry_line_panel, "entry_line_panel", _("Entry line")),
         ]
 
@@ -218,11 +237,14 @@ class MainWindow(wx.Frame, EventMixin):
                           Name("macro_toolbar").Caption(_("Macro toolbar")).
                           Gripper(True).ToolbarPane().Top().Row(1))
 
+        self._mgr.AddPane(self.widget_toolbar, aui.AuiPaneInfo().
+                          Name("widget_toolbar").Caption(_("Widget toolbar")).
+                          Gripper(True).ToolbarPane().Top().Row(1))
+
         self._mgr.AddPane(self.entry_line_panel, aui.AuiPaneInfo().
                           Name("entry_line_panel").Caption(_("Entry line")).
-                          MinSize((10, 20)).Row(2).CaptionVisible(False).
-                          Top().CloseButton(True).Gripper(True).
-                          MaximizeButton(True))
+                          Gripper(False).CenterPane().Top().Row(2).
+                          BestSize(400, 30).PaneBorder(False))
 
         # Load perspective from config
         window_layout = config["window_layout"]
@@ -241,6 +263,15 @@ class MainWindow(wx.Frame, EventMixin):
 
         self._set_menu_toggles()
 
+        # Set initial size to config value
+        self.SetInitialSize(config["window_size"])
+        self.SetMinSize((10, 10))
+
+        # TODO: Set window position fix --> different positions for
+        # different window managers prevent this
+
+        self.SetPosition(config["window_position"])
+
     def _bind(self):
         """Bind events to handlers"""
 
@@ -264,6 +295,7 @@ class MainWindow(wx.Frame, EventMixin):
         # Preferences events
 
         self.Bind(self.EVT_CMD_PREFERENCES, handlers.OnPreferences)
+        self.Bind(self.EVT_CMD_NEW_GPG_KEY, handlers.OnNewGpgKey)
 
         # Toolbar toggle events
 
@@ -271,6 +303,8 @@ class MainWindow(wx.Frame, EventMixin):
                   handlers.OnMainToolbarToggle)
         self.Bind(self.EVT_CMD_MACROTOOLBAR_TOGGLE,
                   handlers.OnMacroToolbarToggle)
+        self.Bind(self.EVT_CMD_WIDGETTOOLBAR_TOGGLE,
+                  handlers.OnWidgetToolbarToggle)
         self.Bind(self.EVT_CMD_ATTRIBUTESTOOLBAR_TOGGLE,
                   handlers.OnAttributesToolbarToggle)
         self.Bind(self.EVT_CMD_FIND_TOOLBAR_TOGGLE,
@@ -287,6 +321,7 @@ class MainWindow(wx.Frame, EventMixin):
         self.Bind(self.EVT_CMD_SAVEAS, handlers.OnSaveAs)
         self.Bind(self.EVT_CMD_IMPORT, handlers.OnImport)
         self.Bind(self.EVT_CMD_EXPORT, handlers.OnExport)
+        self.Bind(self.EVT_CMD_EXPORT_PDF, handlers.OnExportPDF)
         self.Bind(self.EVT_CMD_APPROVE, handlers.OnApprove)
         self.Bind(self.EVT_CMD_CLEAR_GLOBALS, handlers.OnClearGlobals)
 
@@ -359,10 +394,9 @@ class MainWindowEventHandlers(EventMixin):
         """Main window move event"""
 
         # Store window position in config
+        position = self.main_window.GetScreenPositionTuple()
 
-        position = event.GetPosition()
-
-        config["window_position"] = repr((position.x, position.y))
+        config["window_position"] = repr(position)
 
     def OnSize(self, event):
         """Main window move event"""
@@ -414,7 +448,7 @@ class MainWindowEventHandlers(EventMixin):
 
         # Run macros
 
-        ##self.MainGrid.model.pysgrid.sgrid.execute_macros(safe_mode=False)
+        # self.MainGrid.model.pysgrid.sgrid.execute_macros(safe_mode=False)
 
         # Disable menu item for leaving safe mode
 
@@ -479,6 +513,23 @@ class MainWindowEventHandlers(EventMixin):
                 else:
                     config[key] = ast.literal_eval(preferences[key])
 
+    def OnNewGpgKey(self, event):
+        """New GPG key event handler.
+
+        Launches GPG choice and creation dialog
+
+        """
+
+        if genkey is None:
+            # gnupg is not present
+            self.interfaces.display_warning(
+                _("Python gnupg not found. No key selected."),
+                _("Key selection failed."))
+        else:
+            # gnupg is present
+            config["gpg_key_fingerprint"] = repr("")
+            genkey()
+
     # Toolbar events
 
     def _toggle_pane(self, pane):
@@ -520,6 +571,16 @@ class MainWindowEventHandlers(EventMixin):
 
         event.Skip()
 
+    def OnWidgetToolbarToggle(self, event):
+        """Widget toolbar toggle event handler"""
+
+        self.main_window.widget_toolbar.SetGripperVisible(True)
+        widget_toolbar_info = self.main_window._mgr.GetPane("widget_toolbar")
+
+        self._toggle_pane(widget_toolbar_info)
+
+        event.Skip()
+
     def OnAttributesToolbarToggle(self, event):
         """Format toolbar toggle event handler"""
 
@@ -663,9 +724,14 @@ class MainWindowEventHandlers(EventMixin):
 
         # Get filepath from user
 
-        try:
+        if xlrd is None:
             # Do not offer xls if xlrd is missing
-            import xlrd
+            wildcard = \
+                _("Pyspread file") + " (*.pys)|*.pys|" + \
+                _("Uncompressed pyspread file") + " (*.pysu)|*.pysu|" + \
+                _("All files") + " (*.*)|*.*"
+            filetypes = ["pys", "pysu", "pys"]
+        else:
             wildcard = \
                 _("Pyspread file") + " (*.pys)|*.pys|" + \
                 _("Uncompressed pyspread file") + " (*.pysu)|*.pysu|" + \
@@ -673,13 +739,6 @@ class MainWindowEventHandlers(EventMixin):
                 _("All files") + " (*.*)|*.*"
             filetypes = ["pys", "pysu", "xls", "pys"]
 
-        except ImportError:
-            wildcard = \
-                _("Pyspread file") + " (*.pys)|*.pys|" + \
-                _("Uncompressed pyspread file") + " (*.pysu)|*.pysu|" + \
-                _("All files") + " (*.*)|*.*"
-            filetypes = ["pys", "pysu", "pys"]
-
         message = _("Choose file to open.")
         style = wx.OPEN
         filepath, filterindex = \
@@ -729,7 +788,12 @@ class MainWindowEventHandlers(EventMixin):
             filetype = event.attr["filetype"]
 
         except (KeyError, AttributeError):
-            filetype = "pys"
+            if self.main_window.filepath is None:
+                filetype = "pys"
+            elif self.main_window.filepath.endswith("pysu"):
+                filetype = "pysu"
+            else:
+                filetype = "pys"
 
         # If there is no filepath then jump to save as
 
@@ -864,6 +928,8 @@ class MainWindowEventHandlers(EventMixin):
 
         """
 
+        filters = []
+
         code_array = self.main_window.grid.code_array
         tab = self.main_window.grid.current_table
 
@@ -874,6 +940,13 @@ class MainWindowEventHandlers(EventMixin):
         selection_bbox = selection.get_bbox()
 
         wildcard = _("CSV file") + " (*.*)|*.*"
+        filters.append("csv")
+
+        if cairo is not None:
+            wildcard += "|" + _("PDF file") + " (*.pdf)|*.pdf"
+            filters.append("pdf")
+            wildcard += "|" + _("SVG file") + " (*.svg)|*.svg"
+            filters.append("svg")
 
         if selection_bbox is None:
             # No selection --> Use smallest filled area for bottom right edge
@@ -906,13 +979,18 @@ class MainWindowEventHandlers(EventMixin):
         if selection_bbox is None:
             cursor = self.main_window.grid.actions.cursor
             figure = code_array[cursor]
-            if isinstance(figure, Figure):
+            if Figure is not None and isinstance(figure, Figure):
                 wildcard += \
-                    "|" + _("SVG file") + " (*.svg)|*.svg" + \
-                    "|" + _("EPS file") + " (*.eps)|*.eps" + \
-                    "|" + _("PS file") + " (*.ps)|*.ps" + \
-                    "|" + _("PDF file") + " (*.pdf)|*.pdf" + \
-                    "|" + _("PNG file") + " (*.png)|*.png"
+                    "|" + _("SVG of current cell") + " (*.svg)|*.svg" + \
+                    "|" + _("EPS of current cell") + " (*.eps)|*.eps" + \
+                    "|" + _("PS of current cell") + " (*.ps)|*.ps" + \
+                    "|" + _("PDF of current cell") + " (*.pdf)|*.pdf" + \
+                    "|" + _("PNG of current cell") + " (*.png)|*.png"
+                filters.append("cell_svg")
+                filters.append("cell_eps")
+                filters.append("cell_ps")
+                filters.append("cell_pdf")
+                filters.append("cell_png")
 
         message = _("Choose filename for export.")
         style = wx.SAVE
@@ -920,17 +998,36 @@ class MainWindowEventHandlers(EventMixin):
             self.interfaces.get_filepath_findex_from_user(wildcard, message,
                                                           style)
 
-        # If an svg is exported then the selection bbox
+        # If an single cell is exported then the selection bbox
         # has to be changed to the current cell
-        if filterindex >= 1:
+        if filters[filterindex].startswith("cell_"):
             data = figure
 
         # Export file
         # -----------
 
-        self.main_window.actions.export_file(path, filterindex, data,
+        self.main_window.actions.export_file(path, filters[filterindex], data,
                                              preview_data)
 
+    def OnExportPDF(self, event):
+        """Export PDF event handler"""
+
+        # Get filepath from user
+        wildcard = _("PDF") + " (*.pdf)|*.pdf"
+        message = _("Choose file path for PDF export.")
+
+        style = wx.SAVE
+        filepath, filterindex = \
+            self.interfaces.get_filepath_findex_from_user(wildcard, message,
+                                                          style)
+
+        if filepath is None:
+            return
+
+        self.main_window.actions.export_cairo(filepath, "pdf")
+
+        event.Skip()
+
     def OnApprove(self, event):
         """File approve event handler"""
 
@@ -1305,4 +1402,4 @@ class MainWindowEventHandlers(EventMixin):
 
         self.main_window.interfaces.display_about(self.main_window)
 
-# End of class MainWindowEventHandlers
\ No newline at end of file
+# End of class MainWindowEventHandlers
