@@ -38,7 +38,7 @@ from _events import post_command_event, EventMixin
 
 import src.lib.i18n as i18n
 
-#use ugettext instead of getttext to avoid unicode errors
+# Use ugettext instead of getttext to avoid unicode errors
 _ = i18n.language.ugettext
 
 
@@ -211,6 +211,9 @@ class MainMenu(_filledMenu, EventMixin):
                 [item, [self.PreferencesMsg, _("Preferences..."),
                         _("Change preferences of pyspread"),
                         wx.ID_PREFERENCES]],
+                [item, [self.NewGpgKeyMsg, _("Switch GPG key..."),
+                        _("Create or choose a GPG key pair for signing and "
+                          "verifying pyspread files")]],
                 ["Separator"],
                 [item, [self.CloseMsg, _("&Quit") + "\tCtrl+q",
                         _("Quit pyspread"), wx.ID_EXIT]]]],
@@ -277,6 +280,9 @@ class MainMenu(_filledMenu, EventMixin):
                     [item, [self.MacroToolbarToggleMsg, _("Macro toolbar"),
                             _("Shows and hides the macro toolbar.")],
                         wx.ITEM_CHECK],
+                    [item, [self.WidgetToolbarToggleMsg, _("Widget toolbar"),
+                            _("Shows and hides the widget toolbar.")],
+                        wx.ITEM_CHECK],
                     [item, [self.AttributesToolbarToggleMsg,
                             _("Format toolbar"),
                             _("Shows and hides the format toolbar.")],
