@@ -27,9 +27,12 @@ Provides toolbars
 
 Provides:
 ---------
-  1. MainToolbar: Main toolbar of pyspread
-  2. FindToolbar: Toolbar for Find operation
-  3. AttributesToolbar: Toolbar for editing cell attributes
+  1. ToolbarBase: Toolbat base class
+  2. MainToolbar: Main toolbar of pyspread
+  3. MacroToolbar: Shortcuts to common macros
+  4. WidgetToolbar: Make cells behave like widgets
+  5. FindToolbar: Toolbar for Find operation
+  6. AttributesToolbar: Toolbar for editing cell attributes
 
 """
 
@@ -47,7 +50,7 @@ from icons import icons
 
 import _widgets
 
-#use ugettext instead of getttext to avoid unicode errors
+# Use ugettext instead of getttext to avoid unicode errors
 _ = i18n.language.ugettext
 
 
@@ -66,10 +69,16 @@ class ToolbarBase(aui.AuiToolBar, EventMixin):
 
     """
 
+    # Toolbars should be able to overflow
+    style = aui.AUI_TB_OVERFLOW | aui.AUI_TB_GRIPPER | \
+        aui.AUI_TB_PLAIN_BACKGROUND
+
     def __init__(self, parent, *args, **kwargs):
 
-        # Toolbars should be able to overflow
-        kwargs["agwStyle"] = aui.AUI_TB_OVERFLOW | aui.AUI_TB_GRIPPER
+        try:
+            kwargs["agwStyle"] |= self.style
+        except KeyError:
+            kwargs["agwStyle"] = self.style
 
         aui.AuiToolBar.__init__(self, parent, *args, **kwargs)
 
@@ -152,6 +161,7 @@ class MainToolbar(ToolbarBase):
             ["T", self.NewMsg, "FileNew", _("New")],
             ["T", self.OpenMsg, "FileOpen", _("Open")],
             ["T", self.SaveMsg, "FileSave", _("Save")],
+            ["T", self.ExportPDFMsg, "ExportPDF", _("Export PDF")],
             ["S"],
             ["T", self.UndoMsg, "Undo", _("Undo")],
             ["T", self.RedoMsg, "Redo", _("Redo")],
@@ -194,6 +204,77 @@ class MacroToolbar(ToolbarBase):
 # end of class MainToolbar
 
 
+class WidgetToolbar(ToolbarBase):
+    """Widget toolbar, built from attribute toolbardata"""
+
+    def __init__(self, parent, *args, **kwargs):
+
+        ToolbarBase.__init__(self, parent, *args, **kwargs)
+
+        self.button_cell_button_id = wx.NewId()
+        iconname = "BUTTON_CELL"
+        bmp = icons[iconname]
+        self.AddCheckTool(self.button_cell_button_id, iconname, bmp, bmp,
+                          short_help_string=_("Button like cell"))
+        self.Bind(wx.EVT_TOOL, self.OnButtonCell,
+                  id=self.button_cell_button_id)
+
+        self.parent.Bind(self.EVT_CMD_TOOLBAR_UPDATE, self.OnUpdate)
+
+    def _get_button_label(self):
+        """Gets Button label from user and returns string"""
+
+        dlg = wx.TextEntryDialog(self, _('Button label:'))
+
+        if dlg.ShowModal() == wx.ID_OK:
+            label = dlg.GetValue()
+        else:
+            label = ""
+
+        dlg.Destroy()
+
+        return label
+
+    def OnButtonCell(self, event):
+        """Event handler for cell button toggle button"""
+
+        if self.button_cell_button_id == event.GetId():
+            if event.IsChecked():
+                label = self._get_button_label()
+                post_command_event(self, self.ButtonCellMsg, text=label)
+            else:
+                post_command_event(self, self.ButtonCellMsg, text=False)
+
+        event.Skip()
+
+    def OnUpdate(self, event):
+        """Updates the toolbar states"""
+
+        attributes = event.attr
+
+        self._update_buttoncell(attributes["button_cell"])
+
+        self.Refresh()
+
+        event.Skip()
+
+    def _update_buttoncell(self, button_cell):
+        """Updates button cell widget
+
+        Parameters
+        ----------
+
+        button_cell: Bool or string
+        \tUntoggled iif False
+
+        """
+
+        self.ToggleTool(self.button_cell_button_id, button_cell)
+
+
+# end of class WidgetToolbar
+
+
 class FindToolbar(ToolbarBase):
     """Toolbar for find operations (replaces wxFindReplaceDialog)"""
 
@@ -323,7 +404,7 @@ class FindToolbar(ToolbarBase):
 # end of class FindToolbar
 
 
-class AttributesToolbar(aui.AuiToolBar, EventMixin):
+class AttributesToolbar(ToolbarBase, EventMixin):
     """Toolbar for editing cell attributes
 
     Class attributes
@@ -356,8 +437,9 @@ class AttributesToolbar(aui.AuiToolBar, EventMixin):
     }
 
     def __init__(self, parent, *args, **kwargs):
-        kwargs["style"] = aui.AUI_TB_OVERFLOW
-        aui.AuiToolBar.__init__(self, parent, *args, **kwargs)
+        self.style |= aui.AUI_TB_OVERFLOW
+        kwargs["style"] = self.style
+        ToolbarBase.__init__(self, parent, *args, **kwargs)
 
         self.parent = parent
 
@@ -366,13 +448,13 @@ class AttributesToolbar(aui.AuiToolBar, EventMixin):
         self._create_font_choice_combo()
         self._create_font_size_combo()
         self._create_font_face_buttons()
+        self._create_textrotation_button()
         self._create_justification_button()
         self._create_alignment_button()
         self._create_borderchoice_combo()
         self._create_penwidth_combo()
         self._create_color_buttons()
         self._create_merge_button()
-        self._create_textrotation_spinctrl()
 
         self.Realize()
 
@@ -426,6 +508,7 @@ class AttributesToolbar(aui.AuiToolBar, EventMixin):
                 "FormatTextStrikethrough", _("Strikethrough")),
             (wx.FONTFLAG_MASK, "OnFreeze", "Freeze", _("Freeze")),
             (wx.FONTFLAG_NOT_ANTIALIASED, "OnLock", "Lock", _("Lock cell")),
+            (wx.FONTFAMILY_DECORATIVE, "OnMarkup", "Markup", _("Markup")),
         ]
 
         for __id, method, iconname, helpstring in font_face_buttons:
@@ -434,6 +517,18 @@ class AttributesToolbar(aui.AuiToolBar, EventMixin):
                               short_help_string=helpstring)
             self.Bind(wx.EVT_TOOL, getattr(self, method), id=__id)
 
+    def _create_textrotation_button(self):
+        """Create text rotation toggle button"""
+
+        iconnames = ["TextRotate270", "TextRotate0", "TextRotate90",
+                     "TextRotate180"]
+        bmplist = [icons[iconname] for iconname in iconnames]
+
+        self.rotation_tb = _widgets.BitmapToggleButton(self, bmplist)
+        self.rotation_tb.SetToolTipString(_(u"Cell text rotation"))
+        self.Bind(wx.EVT_BUTTON, self.OnRotate, self.rotation_tb)
+        self.AddControl(self.rotation_tb)
+
     def _create_justification_button(self):
         """Creates horizontal justification button"""
 
@@ -531,21 +626,6 @@ class AttributesToolbar(aui.AuiToolBar, EventMixin):
                           short_help_string=_("Merge cells"))
         self.Bind(wx.EVT_TOOL, self.OnMerge, id=self.mergetool_id)
 
-    def _create_textrotation_spinctrl(self):
-        """Create text rotation spin control"""
-
-        self.rotation_spinctrl = wx.SpinCtrl(self, -1, "", size=(50, -1))
-        self.rotation_spinctrl.SetRange(-179, 180)
-        self.rotation_spinctrl.SetValue(0)
-
-        # For compatibility with toggle buttons
-        self.rotation_spinctrl.GetToolState = lambda x: None
-        self.rotation_spinctrl.SetToolTipString(_(u"Cell text rotation"))
-
-        self.AddControl(self.rotation_spinctrl)
-
-        self.Bind(wx.EVT_SPINCTRL, self.OnRotate, self.rotation_spinctrl)
-
     # Update widget state methods
     # ---------------------------
 
@@ -621,7 +701,7 @@ class AttributesToolbar(aui.AuiToolBar, EventMixin):
 
         """
 
-        toggle_state = not frozen is False
+        toggle_state = frozen is not False
 
         self.ToggleTool(wx.FONTFLAG_MASK, toggle_state)
 
@@ -638,6 +718,19 @@ class AttributesToolbar(aui.AuiToolBar, EventMixin):
 
         self.ToggleTool(wx.FONTFLAG_NOT_ANTIALIASED, locked)
 
+    def _update_markupcell(self, markup):
+        """Updates markup cell widget
+
+        Parameters
+        ----------
+
+        markup: Bool
+        \tUntoggled iif False
+
+        """
+
+        self.ToggleTool(wx.FONTFAMILY_DECORATIVE, markup)
+
     def _update_underline(self, underlined):
         """Updates underline widget
 
@@ -664,6 +757,19 @@ class AttributesToolbar(aui.AuiToolBar, EventMixin):
 
         self.ToggleTool(wx.FONTFLAG_STRIKETHROUGH, strikethrough)
 
+    def _update_textrotation(self, angle):
+        """Updates text rotation toggle button"""
+
+        states = {0: 0, -90: 1, 180: 2, 90: 3}
+
+        try:
+            self.rotation_tb.state = states[round(angle)]
+        except KeyError:
+            self.rotation_tb.state = 0
+
+        self.rotation_tb.toggle(None)
+        self.rotation_tb.Refresh()
+
     def _update_justification(self, justification):
         """Updates horizontal text justification button
 
@@ -721,11 +827,6 @@ class AttributesToolbar(aui.AuiToolBar, EventMixin):
 
         self.ToggleTool(self.mergetool_id, merged)
 
-    def _update_textrotation(self, angle):
-        """Updates text rotation spin control"""
-
-        self.rotation_spinctrl.SetValue(angle)
-
     def _update_bgbrush(self, bgcolor):
         """Updates background color"""
 
@@ -761,6 +862,7 @@ class AttributesToolbar(aui.AuiToolBar, EventMixin):
         self._update_font_style(attributes["fontstyle"])
         self._update_frozencell(attributes["frozen"])
         self._update_lockedcell(attributes["locked"])
+        self._update_markupcell(attributes["markup"])
         self._update_underline(attributes["underline"])
         self._update_strikethrough(attributes["strikethrough"])
         self._update_justification(attributes["justification"])
@@ -774,6 +876,8 @@ class AttributesToolbar(aui.AuiToolBar, EventMixin):
 
         self.Refresh()
 
+        event.Skip()
+
     def OnBorderChoice(self, event):
         """Change the borders that are affected by color and width changes"""
 
@@ -868,6 +972,11 @@ class AttributesToolbar(aui.AuiToolBar, EventMixin):
 
         post_command_event(self, self.LockMsg)
 
+    def OnMarkup(self, event):
+        """Markup toggle button event handler"""
+
+        post_command_event(self, self.MarkupMsg)
+
     def OnJustification(self, event):
         """Justification toggle button event handler"""
 
@@ -886,8 +995,6 @@ class AttributesToolbar(aui.AuiToolBar, EventMixin):
     def OnRotate(self, event):
         """Rotation spin control event handler"""
 
-        angle = self.rotation_spinctrl.GetValue()
-
-        post_command_event(self, self.TextRotationMsg, angle=angle)
+        post_command_event(self, self.TextRotationMsg)
 
 # end of class AttributesToolbar
