@@ -43,6 +43,7 @@ Provides:
 import __builtin__
 import keyword
 from copy import copy
+import time
 
 try:
     import jedi
@@ -57,8 +58,10 @@ from wx.lib.intctrl import IntCtrl, EVT_INT
 
 import src.lib.i18n as i18n
 from src.lib.parsers import common_start
+from src.lib._string_helpers import quote
+
 from src.config import config
-from src.sysvars import get_default_font, is_gtk
+from src.sysvars import get_default_font, is_gtk, get_color
 
 from _events import post_command_event, EntryLineEventMixin, GridCellEventMixin
 from _events import StatusBarEventMixin, GridEventMixin, GridActionEventMixin
@@ -590,6 +593,10 @@ class FontChoiceCombobox(ImageComboBox):
 class BorderEditChoice(ImageComboBox):
     """Combo box for selecting the cell borders that shall be changed"""
 
+    def __init__(self, *args, **kwargs):
+        ImageComboBox.__init__(self, *args, **kwargs)
+        self.SetSelection(0)
+
     def OnDrawItem(self, dc, rect, item, flags):
 
         if item == wx.NOT_FOUND:
@@ -681,10 +688,16 @@ class EntryLineToolbarPanel(wx.Panel):
 
     def __init__(self, parent, *args, **kwargs):
         wx.Panel.__init__(self, parent, *args, **kwargs)
+        try:
+            self.SetBackgroundColour(get_color(wx.SYS_COLOUR_FRAMEBK))
+        except AttributeError:
+            # Does not work on wx 2.x
+            pass
 
         self.parent = parent
         # Panel with EntryLine and button
-        self.entry_line_panel = EntryLinePanel(self, parent, -1)
+        self.entry_line_panel = EntryLinePanel(self, parent,
+                                               style=wx.NO_BORDER)
 
         # IntCtrl for table choice
         self.table_choice = TableChoiceIntCtrl(self, parent,
@@ -718,6 +731,12 @@ class EntryLinePanel(wx.Panel, GridEventMixin, GridActionEventMixin):
 
     def __init__(self, parent, main_window, *args, **kwargs):
         wx.Panel.__init__(self, parent, *args, **kwargs)
+
+        try:
+            self.SetBackgroundColour(get_color(wx.SYS_COLOUR_FRAMEBK))
+        except AttributeError:
+            # Does not work on wx 2.x
+            pass
         self.parent = parent
         self.main_window = main_window
 
@@ -739,8 +758,8 @@ class EntryLinePanel(wx.Panel, GridEventMixin, GridActionEventMixin):
     def __do_layout(self):
         main_sizer = wx.FlexGridSizer(1, 2, 0, 0)
 
-        main_sizer.Add(self.entry_line, 1, wx.ALL | wx.EXPAND, 1)
-        main_sizer.Add(self.selection_toggle_button, 1, wx.ALL | wx.EXPAND, 1)
+        main_sizer.Add(self.entry_line, 0, wx.EXPAND, 0)
+        main_sizer.Add(self.selection_toggle_button, 0, wx.EXPAND, 0)
 
         main_sizer.AddGrowableRow(0)
         main_sizer.AddGrowableCol(0)
@@ -775,11 +794,9 @@ class EntryLine(wx.TextCtrl, EntryLineEventMixin, GridCellEventMixin,
 
     def __init__(self, parent, main_window, id=-1, *args, **kwargs):
         kwargs["style"] = wx.TE_PROCESS_ENTER | wx.TE_MULTILINE | \
-            wx.TE_PROCESS_TAB
+            wx.TE_PROCESS_TAB | wx.NO_BORDER
         wx.TextCtrl.__init__(self, parent, id, *args, **kwargs)
 
-        self.SetSize((700, 25))
-
         self.parent = parent
         self.main_window = main_window
         self.ignore_changes = False
@@ -794,12 +811,25 @@ class EntryLine(wx.TextCtrl, EntryLineEventMixin, GridCellEventMixin,
         main_window.Bind(self.EVT_CMD_SELECTION, self.OnGridSelection)
         main_window.Bind(self.EVT_ENTRYLINE_LOCK, self.OnLock)
 
+        self.Bind(wx.EVT_SET_FOCUS, self.OnFocus)
+        self.Bind(wx.EVT_KILL_FOCUS, self.OnKillFocus)
+
         self.SetToolTip(wx.ToolTip(_("Enter Python expression here.")))
 
         self.Bind(wx.EVT_TEXT, self.OnText)
         self.Bind(wx.EVT_CHAR, self.OnChar)
         self.main_window.Bind(self.EVT_CMD_TABLE_CHANGED, self.OnTableChanged)
 
+    def OnFocus(self, event):
+        """SetFocus event handler"""
+
+        event.Skip()
+
+    def OnKillFocus(self, event):
+        """KillFocus event handler"""
+
+        event.Skip()
+
     def OnContentChange(self, event):
         """Event handler for updating the content"""
 
@@ -846,6 +876,8 @@ class EntryLine(wx.TextCtrl, EntryLineEventMixin, GridCellEventMixin,
         if not self.ignore_changes:
             post_command_event(self, self.CodeEntryMsg, code=event.GetString())
 
+        self.main_window.grid.grid_renderer.cell_cache.clear()
+
         event.Skip()
 
     def OnChar(self, event):
@@ -867,7 +899,7 @@ class EntryLine(wx.TextCtrl, EntryLineEventMixin, GridCellEventMixin,
 
                 # Ignore <Ctrl> + <Enter> and Quote content
                 if event.ControlDown():
-                    self.SetValue('"' + self.GetValue() + '"')
+                    self.SetValue(quote(self.GetValue()))
 
                 # Do not process <Enter>
                 return
@@ -921,7 +953,7 @@ class EntryLine(wx.TextCtrl, EntryLineEventMixin, GridCellEventMixin,
                 docs = []
                 for completion in completions:
                     doc = completion.docstring(fast=False)
-                    if not doc:
+                    if not doc and code:
                         # Is the completion part of a module?
                         code_segment = \
                             code[:position+1].split()[-1]
@@ -1053,17 +1085,16 @@ class StatusBar(wx.StatusBar, StatusBarEventMixin, MainWindowEventMixin):
 
 
 class TableChoiceIntCtrl(IntCtrl, GridEventMixin, GridActionEventMixin):
-    """ComboBox for choosing the current grid table"""
+    """IntCtrl for choosing the current grid table"""
 
     def __init__(self, parent, main_window, no_tabs):
         self.parent = parent
         self.main_window = main_window
         self.no_tabs = no_tabs
 
-        IntCtrl.__init__(self, parent, limited=True, allow_long=True)
+        IntCtrl.__init__(self, parent, allow_long=True, style=wx.NO_BORDER)
 
-        self.SetMin(0)
-        self.SetMax(no_tabs - 1)
+        self.last_change_s = time.clock()
 
         tipmsg = _("For switching tables enter the table number or "
                    "use the mouse wheel.")
@@ -1071,9 +1102,11 @@ class TableChoiceIntCtrl(IntCtrl, GridEventMixin, GridActionEventMixin):
 
         # State for preventing to post GridActionTableSwitchMsg
         self.switching = False
+        self.cursor_pos = 0
 
         self.Bind(EVT_INT, self.OnInt)
         self.Bind(wx.EVT_MOUSEWHEEL, self.OnMouseWheel)
+        self.Bind(wx.EVT_SET_FOCUS, self.OnFocus)
         self.main_window.Bind(self.EVT_CMD_RESIZE_GRID, self.OnResizeGrid)
         self.main_window.Bind(self.EVT_CMD_TABLE_CHANGED, self.OnTableChanged)
 
@@ -1094,8 +1127,6 @@ class TableChoiceIntCtrl(IntCtrl, GridEventMixin, GridActionEventMixin):
         if self.GetValue() >= no_tabs:
             self.SetValue(no_tabs - 1)
 
-        self.SetMax(no_tabs - 1)
-
     # Event handlers
 
     def OnResizeGrid(self, event):
@@ -1103,25 +1134,59 @@ class TableChoiceIntCtrl(IntCtrl, GridEventMixin, GridActionEventMixin):
 
         self.change_max(event.shape[2])
 
+    def _fromGUI(self, value):
+        """
+        Conversion function used in getting the value of the control.
+
+        """
+
+        # One or more of the underlying text control implementations
+        # issue an intermediate EVT_TEXT when replacing the control's
+        # value, where the intermediate value is an empty string.
+        # So, to ensure consistency and to prevent spurious ValueErrors,
+        # we make the following test, and react accordingly:
+        #
+        if value == '':
+            if not self.IsNoneAllowed():
+                return 0
+            else:
+                return
+        else:
+            try:
+                return int(value)
+            except ValueError:
+                if self.IsLongAllowed():
+                    try:
+                        return long(value)
+                    except ValueError:
+                        wx.TextCtrl.SetValue(self, "0")
+                        return 0
+                else:
+                    raise
+
     def OnInt(self, event):
         """IntCtrl event method that updates the current table"""
 
-        self.SetMax(self.no_tabs - 1)
-        if event.GetValue() > self.GetMax():
-            self.SetValue(self.GetMax())
+        value = event.GetValue()
+
+        current_time = time.clock()
+        if current_time < self.last_change_s + 0.01:
             return
+        self.last_change_s = current_time
 
-        if not self.switching:
-            self.switching = True
-            post_command_event(self, self.GridActionTableSwitchMsg,
-                               newtable=event.GetValue())
-            if is_gtk():
-                try:
-                    wx.Yield()
-                except:
-                    pass
+        self.cursor_pos = wx.TextCtrl.GetInsertionPoint(self) + 1
 
-            self.switching = False
+        if event.GetValue() > self.no_tabs - 1:
+            value = self.no_tabs - 1
+
+        self.switching = True
+        post_command_event(self, self.GridActionTableSwitchMsg, newtable=value)
+        self.switching = False
+
+    def OnFocus(self, event):
+        """Focus event handler"""
+
+        wx.TextCtrl.SetInsertionPoint(self, self.cursor_pos)
 
     def OnMouseWheel(self, event):
         """Mouse wheel event handler"""
@@ -1130,15 +1195,16 @@ class TableChoiceIntCtrl(IntCtrl, GridEventMixin, GridActionEventMixin):
         if self.switching:
             return
 
-        self.SetMax(self.no_tabs - 1)
+        value = self.GetValue()
+
+        current_time = time.clock()
+        if current_time < self.last_change_s + 0.01:
+            return
 
         if event.GetWheelRotation() > 0:
-            new_table = self.GetValue() + 1
+            self.SetValue(min(value+1, self.no_tabs-1))
         else:
-            new_table = self.GetValue() - 1
-
-        if self.IsInBounds(new_table):
-            self.SetValue(new_table)
+            self.SetValue(max(value-1, 0))
 
     def OnShapeChange(self, event):
         """Grid shape change event handler"""
@@ -1153,6 +1219,8 @@ class TableChoiceIntCtrl(IntCtrl, GridEventMixin, GridActionEventMixin):
         if hasattr(event, 'table'):
             self.SetValue(event.table)
 
+        wx.TextCtrl.SetInsertionPoint(self, self.cursor_pos)
+
         event.Skip()
 
 # end of class TableChoiceIntCtrl
