@@ -66,6 +66,7 @@ class GtkArtProvider(wx.ArtProvider):
             "AlignBottom": action_path + "format-text-alignbottom.png",
             "Freeze": action_path + "frozen_small.png",
             "Lock": action_path + "lock.png",
+            "Markup": action_path + "format_text_markup.png",
             "Merge": action_path + "format-merge-table-cells.png",
             "AllBorders": toggle_path + "border_all.xpm",
             "LeftBorders": toggle_path + "border_left.xpm",
@@ -92,6 +93,12 @@ class GtkArtProvider(wx.ArtProvider):
             "safe_mode": icon_path + "status/dialog-warning.png",
             "SortAscending": action_path + "edit-sort-ascending.png",
             "SortDescending": action_path + "edit-sort-descending.png",
+            "ExportPDF": action_path + "export_pdf.png",
+            "TextRotate0": action_path + "format-text-rotate0.png",
+            "TextRotate90": action_path + "format-text-rotate90.png",
+            "TextRotate180": action_path + "format-text-rotate180.png",
+            "TextRotate270": action_path + "format-text-rotate270.png",
+            "BUTTON_CELL": action_path + "button_ok.png",
         }
 
     def get_paths(self, theme, icon_size):
