@@ -35,17 +35,14 @@ Provides
 
 """
 
-import sys
-
 import wx
-import wx.lib.agw.genericmessagedialog as GMD
 import gnupg
 
 import src.lib.i18n as i18n
 from src.config import config
 from src.gui._gui_interfaces import get_key_params_from_user
 
-#use ugettext instead of getttext to avoid unicode errors
+# Use ugettext instead of getttext to avoid unicode errors
 _ = i18n.language.ugettext
 
 
@@ -83,8 +80,36 @@ def choose_key(gpg_private_keys, gpg_private_fingerprints):
     return key
 
 
-def genkey():
-    """Creates a new standard GPG key"""
+def _register_key(fingerprint, gpg):
+    """Registers key in config"""
+
+    for private_key in gpg.list_keys(True):
+        try:
+            if str(fingerprint) == private_key['fingerprint']:
+                config["gpg_key_fingerprint"] = \
+                    repr(private_key['fingerprint'])
+        except KeyError:
+            pass
+
+
+def genkey(key_name=None):
+    """Creates a new standard GPG key
+
+    Parameters
+    ----------
+
+    ui: Bool
+    \tIf True, then a new key is created when required without user interaction
+
+    """
+
+    gpg_key_param_list = [
+        ('key_type', 'DSA'),
+        ('key_length', '2048'),
+        ('subkey_type', 'ELG-E'),
+        ('subkey_length', '2048'),
+        ('expire_date', '0'),
+    ]
 
     gpg = gnupg.GPG()
 
@@ -92,7 +117,7 @@ def genkey():
 
     # Check if standard key is already present
 
-    pyspread_key_fingerprint = str(config["gpg_key_fingerprint"])
+    pyspread_key_fingerprint = config["gpg_key_fingerprint"]
     gpg_private_keys = gpg.list_keys(True)
     gpg_private_fingerprints = gpg.list_keys(True).fingerprints
 
@@ -100,10 +125,10 @@ def genkey():
 
     for private_key, fingerprint in zip(gpg_private_keys,
                                         gpg_private_fingerprints):
-        if pyspread_key_fingerprint == fingerprint:
+        if str(pyspread_key_fingerprint) == fingerprint:
             pyspread_key = private_key
 
-    if pyspread_key is None:
+    if key_name is None and pyspread_key is None:
         # If no GPG key is set in config, choose one
         pyspread_key = choose_key(gpg_private_keys, gpg_private_fingerprints)
 
@@ -115,37 +140,49 @@ def genkey():
 
     else:
         # No key has been chosen --> Create new one
-        gpg_key_parameters = get_key_params_from_user()
+        if key_name is None:
+            gpg_key_parameters = get_key_params_from_user(gpg_key_param_list)
+            if gpg_key_parameters is None:
+                # No name entered
+                return
+        else:
+            gpg_key_param_list.append(
+                ('name_real', '{key_name}'.format(key_name=key_name)))
+            gpg_key_parameters = dict(gpg_key_param_list)
 
         input_data = gpg.gen_key_input(**gpg_key_parameters)
 
         # Generate key
         # ------------
 
-        # Show infor dialog
+        if key_name is None:
+            # Show information dialog
 
-        style = wx.ICON_INFORMATION | wx.DIALOG_NO_PARENT | wx.OK | wx.CANCEL
-        pyspread_key_uid = gpg_key_parameters["name_real"]
-        short_message = _("New GPG key").format(pyspread_key_uid)
-        message = _("After confirming this dialog, a new GPG key ") + \
-            _("'{key}' will be generated.").format(key=pyspread_key_uid) + \
-            _(" \n \nThis may take some time.\nPlease wait.\n \n") + \
-            _("Canceling this operation exits pyspread.")
-        dlg = GMD.GenericMessageDialog(None, message, short_message, style)
-        dlg.Centre()
+            style = wx.ICON_INFORMATION | wx.DIALOG_NO_PARENT | wx.OK | \
+                wx.CANCEL
+            pyspread_key_uid = gpg_key_parameters["name_real"]
+            short_message = _("New GPG key").format(pyspread_key_uid)
+            message = _("After confirming this dialog, a new GPG key ") + \
+                _("'{key}' will be generated.").format(key=pyspread_key_uid) +\
+                _(" \n \nThis may take some time.\nPlease wait.")
+            dlg = wx.MessageDialog(None, message, short_message, style)
+            dlg.Centre()
 
-        if dlg.ShowModal() == wx.ID_OK:
-            dlg.Destroy()
-            fingerprint = gpg.gen_key(input_data)
-
-            for private_key in gpg.list_keys(True):
-                if str(fingerprint) == private_key['fingerprint']:
-                    config["gpg_key_fingerprint"] = repr(
-                        private_key.fingerprint)
+            if dlg.ShowModal() == wx.ID_OK:
+                dlg.Destroy()
+                gpg_key = gpg.gen_key(input_data)
+                _register_key(gpg_key, gpg)
+                fingerprint = gpg_key.fingerprint
+            else:
+                dlg.Destroy()
+                return
 
         else:
-            dlg.Destroy()
-            sys.exit()
+            gpg_key = gpg.gen_key(input_data)
+            _register_key(gpg_key, gpg)
+            fingerprint = gpg_key.fingerprint
+
+    return fingerprint
 
 
 def sign(filename):
@@ -167,10 +204,7 @@ def verify(sigfilename, filefilename=None):
 
     gpg = gnupg.GPG()
 
-    sigfile = open(sigfilename, "rb")
-
-    verified = gpg.verify_file(sigfile, filefilename)
-
-    sigfile.close()
+    with open(sigfilename, "rb") as sigfile:
+        verified = gpg.verify_file(sigfile, filefilename)
 
     return verified
