@@ -32,9 +32,16 @@ Provides
  * color2code
  * code2color
  * parse_dict_strings
+ * is_svg
 
 """
 
+try:
+    import rsvg
+    import glib
+except ImportError:
+    rsvg = None
+
 import ast
 
 import wx
@@ -88,6 +95,22 @@ def color2code(color):
     return unicode(tuple(i / 255.0 for i in color.Get()))
 
 
+def color_pack2rgb(packed):
+    """Returns r, g, b tuple from packed wx.ColourGetRGB value"""
+
+    r = packed & 255
+    g = (packed & (255 << 8)) >> 8
+    b = (packed & (255 << 16)) >> 16
+
+    return r, g, b
+
+
+def color_rgb2pack(r, g, b):
+    """Returns packed wx.ColourGetRGB value from r, g, b tuple"""
+
+    return r + (g << 8) + (b << 16)
+
+
 def unquote_string(code):
     """Returns a string from code that contains aa repr of the string"""
 
@@ -163,3 +186,25 @@ def common_start(strings):
             return start_string[:-1]
 
     return start_string
+
+
+def is_svg(code):
+    """Checks if code is an svg image
+
+    Parameters
+    ----------
+    code: String
+    \tCode to be parsed in order to check svg complaince
+
+    """
+
+    if rsvg is None:
+        return
+
+    try:
+        rsvg.Handle(data=code)
+
+    except glib.GError:
+        return False
+
+    return True
