@@ -114,6 +114,8 @@ class CellAttributes(list):
         "justification": "left",
         "frozen": False,
         "merge_area": None,
+        "markup": False,
+        "button_cell": False,
     }
 
     # Cache for __getattr__ maps key to tuple of len and attr_dict
@@ -355,6 +357,24 @@ class DataArray(object):
 
     data = property(_get_data, _set_data)
 
+    def get_row_height(self, row, tab):
+        """Returns row height"""
+
+        try:
+            return self.row_heights[(row, tab)]
+
+        except KeyError:
+            return config["default_row_height"]
+
+    def get_col_width(self, col, tab):
+        """Returns column width"""
+
+        try:
+            return self.col_widths[(col, tab)]
+
+        except KeyError:
+            return config["default_col_width"]
+
     # Row and column attributes mask
     # Keys have the format (row, table)
 
@@ -780,7 +800,8 @@ class DataArray(object):
                         new_val = copy(value)
                         new_sel.insert(insertion_point, no_to_insert, axis)
                         # Update merge area if present
-                        if "merge_area" in value:
+                        if "merge_area" in value and \
+                           value["merge_area"] is not None:
                             top, left, bottom, right = value["merge_area"]
                             ma_sel = Selection([(top, left)],
                                                [(bottom, right)], [], [], [])
@@ -928,9 +949,8 @@ class DataArray(object):
             if key not in new_keys and self(key) is not None:
                 self.pop(key, mark_unredo=False)
 
-        if axis in (0, 1):
-            self._adjust_rowcol(deletion_point, -no_to_delete, axis, tab=tab,
-                                mark_unredo=False)
+        self._adjust_rowcol(deletion_point, -no_to_delete, axis, tab=tab,
+                            mark_unredo=False)
         self._adjust_cell_attributes(deletion_point, -no_to_delete, axis,
                                      tab=tab, mark_unredo=False)
 
@@ -1121,8 +1141,12 @@ class CodeArray(DataArray):
         # Flatten helper function
         def nn(val):
             """Returns flat numpy arraz without None values"""
+            try:
+                return numpy.array(filter(None, val.flat))
 
-            return numpy.array(filter(None, val.flat))
+            except AttributeError:
+                # Probably no numpy array
+                return numpy.array(filter(None, val))
 
         # Set up environment for evaluation
 
